package tech.nevets.ubackup.client;

import org.apache.commons.net.ftp.FTPSClient;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.*;

import static tech.nevets.ubackup.client.Main.SYS_CONF;

public class FTPSManager {
    private static final Config FTP_CONF = new Config("./conf/ftpsettings.json",
            "{\n  \"host\": \"ftps.example.com\",\n  \"port\": 990, \n  \"username\": \"john\", \n  \"password\":\"password\" \n}"
            );

    private static SSLContext configureSSLContext() {
        try {
            KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
            InputStream tStore = new FileInputStream("./truststore.p12");
            trustStore.load(tStore, "".toCharArray());
            tStore.close();
            TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
            tmf.init(trustStore);

            SSLContext ctx = SSLContext.getInstance("TLS");
            ctx.init(null, tmf.getTrustManagers(), SecureRandom.getInstanceStrong());

            return ctx;
        } catch (final Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private FTPSClient client;

    public FTPSManager() {
        try {
            client = new FTPSClient(configureSSLContext());
            client.connect(FTP_CONF.getString("host"), FTP_CONF.getInt("port"));
            System.out.println(client.getReplyString());
            client.execPROT("P");
            System.out.println(client.getReplyString());
            client.login(FTP_CONF.getString("username"), FTP_CONF.getString("password"));
            System.out.println(client.getReplyString());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void uploadBackup(InputStream fileStream) {
        try {
            String hostname = SYS_CONF.getString("hostname");
            client.storeFile(
                    "/backups/" + hostname + "/" + hostname + "-" + Utils.getDateFormatted(),
                    fileStream);
            System.out.println(client.getReplyString());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void close() {
        try {
            client.disconnect();
            System.out.println(client.getReplyString());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
